/**
  *  \file planetscentral/format/packer.h
  *  \brief Format: Packer base class
  */
#ifndef C2SERVER_PLANETSCENTRAL_FORMAT_PACKER_H
#define C2SERVER_PLANETSCENTRAL_FORMAT_PACKER_H

#include <cstddef>
#include "util/recode.h"
#include "cpluslib/string.h"
#include "io/io.h"

class IntValue;

class Packer {
 public:
    Packer();
    virtual ~Packer();

    /** Pack structured data into binary.
        Throws runtime_error on error.
        \param data structured data (array, string, whatever)
        \return packed data */
    virtual string_t pack(IntValue* data) = 0;

    /** Unpack binary into structured data.
        Throws runtime_error on error.
        \param binary data
        \return structured data */
    virtual IntValue* unpack(const string_t& data) = 0;

    /** Character transformation. */
    void setCharset(string_t charset);

    void packString(char* out, std::size_t size, string_t str) const;
    string_t packString(const string_t& str) const;
    string_t unpackString(const char* in, std::size_t size) const;
    string_t unpackString(const string_t& str) const;

 private:
    bool is_utf8;
    CharacterSet charset;
};

template<typename T>
void
storeStructure(string_t& out, const T& data)
{
    char packed[T::size];
    storeStructure(packed, data);
    out.append(packed, T::size);
}

template<typename T>
bool
getStructure(const string_t& in, string_t::size_type& pos, T& data)
{
    if (pos < in.size() && in.size() - pos >= T::size) {
        getStructure(&in[pos], data);
        pos += T::size;
        return true;
    } else {
        return false;
    }
}

#endif
