/**
  *  \file planetscentral/host/cron.h
  *  \brief Host: Scheduler
  */
#ifndef C2SERVER_PLANETSCENTRAL_HOST_CRON_H
#define C2SERVER_PLANETSCENTRAL_HOST_CRON_H

#include <list>
#include "cpluslib/types.h"
#include "planetscentral/osutil.h"
#include "planetscentral/dbclient/dbsubtree.h"

class Runner;
class DbClient;
class ReconnectingConnection;

enum CronAction {
    UnknownAction,              /* Waiting for decision */
    NoAction,                   /* Nothing planned for this game */
    HostAction,                 /* Host will run */
    ScheduleChangeAction,       /* Schedule will change */
    MasterAction                /* Master will run */
};

class Cron {
 public:
    struct ScheduleItem {
        CronAction action;
        int32_t gameId;
        int32_t time;
        ScheduleItem(CronAction action, int32_t game, int32_t time)
            : action(action), gameId(game), time(time)
            { }
    };

    Cron(Runner& hostRunner, DbClient& dbConnection, ReconnectingConnection& dbLock);
    void start();
    void handleGameStart(int32_t id);
    void handleGameChange(int32_t id);
    ScheduleItem getNextAction(int32_t gameId);
    void getAllSchedules(std::list<ScheduleItem>& output);

 private:
    // Attributes
    Runner& hostRunner;
    DbClient& dbConnection;
    ReconnectingConnection& dbLock;

    // Communication
    Mutex mutex;
    Semaphore new_command_sem;
    std::list<int32_t> new_commands;

    // Schedule
    std::list<ScheduleItem> schedule;       ///< Future actions.
    std::list<ScheduleItem> overdue;        ///< Due actions. Games in this list are locked if they are MasterAction or HostAction.

    // Entry
    static void* cron(void* self);

    // Utilities
    void generateInitialSchedule();
    void processRequests();
    void moveDueItems();
    void runDueItem(int32_t gameId, std::list<ScheduleItem>& newSchedule);
};

const char* getCronActionName(CronAction a);

#endif
