/**
  *  \file planetscentral/rater/difficulty.h
  *  \brief Rating Game Difficulty
  */
#ifndef C2SERVER_PLANETSCENTRAL_RATER_DIFFICULTY_H
#define C2SERVER_PLANETSCENTRAL_RATER_DIFFICULTY_H

#include "io/dirs.h"
#include "cpluslib/smallset.h"

/** Rating game difficulty. This class computes game difficulty from
    shiplist, host and master configuration. Individual partial
    ratings can also be obtained (i.e. just a shiplist rating).

    Use the addXXX() functions to feed this function data. Any partial
    data can be used. */
class DifficultyRater {
 public:
    enum Rating {
        ShiplistRating,
        MineralRating,
        NativeRating,
        ProductionRating
    };

    DifficultyRater();
    ~DifficultyRater();

    void addShipList(Directory& dir);
    void addConfigDirectory(Directory& dir);
    void addConfigFile(Stream& s);
    void addConfig(string_t name, string_t value);

    bool isRatingKnown(Rating whichone) const;
    double getRating(Rating whichone) const;
    double getTotalRating() const;

 private:
    // Configuration
    typedef int32_t Item[11];
    enum Config {
        Master_CoreRangesUsual,
        Master_CoreRangesAlternate,
        Master_CoreUsualFrequency,
        Master_SurfaceRanges,

        Master_NativeFrequency,
        Master_NativeRanges,
        Master_NativeClansRange,

        Host_ProductionRate,
        Host_MiningRate,
        Host_ColonistTaxRate,
        Host_NativeTaxRate,
        Host_PlayerRace,
        Host_PlayerSpecialMission,
        Host_HissEffectRate
    };
    static const uint_t Config_MAX = Host_HissEffectRate+1;
    SmallSet<Config> config_known;
    Item config_values[Config_MAX];

    // Shiplist rating
    bool shiplist_known;
    int shiplist_average_cost;

    // Other ratings
    double getAverageMinerals() const;
    double getAverageNatives() const;
    double getAverageVPI() const;
};

#endif
